# -*- coding: utf-8 -*-


class DTShellConstants(object):
    commands_remote_url = 'https://github.com/duckietown/duckietown-shell-commands'
    ROOT = '~/.dt-shell/'
    ENV_COMMANDS = 'DTSHELL_COMMANDS'

    DT1_TOKEN_CONFIG_KEY = 'token_dt1'
    CONFIG_DOCKER_USERNAME = 'docker_username'
